<?php

class Search_Controller extends AbstractController
{
	function index()
	{
		$this->load->model('Recipe_Model', 'recipe');
		$this->load->model('Special_Model', 'special');

		$query = Url::get('q'); // search query
		$recipe_search = Url::get('type') == 'recipe';
		$special_search = Url::get('type') == 'special';
		$zipCode = Url::get('zipCode');

		$this->view->assign('user', $this->user->getArray());

		if ($recipe_search) {
			// recipe results.
			$recipes = $this->recipe->getRecipesByName($query);
			$recipes = $this->_mergeAssoc($recipes, $this->recipe->getRecipesByIngredient($query), 'id');

			// get a list of associated specials for recipes.
			$specials = array();
			foreach ($recipes as $r) {
				$s = $this->special->getSpecialsForRecipe($r['id'], $zipCode);
				$specials[$r['id']] = $s;
			}

			$this->view->assign('recipes', $recipes);
			$this->view->assign('associated_specials', $specials);
			$this->view->display('search_recipes.html');

		} else if ($special_search) {
			// special results. (search both by ingredient and special title and special description)
			$specials = $this->special->getSpecialsByName($query);
			$specials = $this->_mergeAssoc($specials, $this->special->getSpecialsByIngredient($query), 'id');

			// get a list of associated recipes for specials.
			$recipes = array();
			foreach ($specials as $s) {
				$r = $this->recipe->getRecipesForSpecial($s['id']);
				$recipes[$s['id']] = $r;
			}

			$this->view->assign('specials', $specials);
			$this->view->assign('associated_recipes', $recipes);
			$this->view->display('search_specials.html');
		} else {
			// default search page.
			$this->view->display('search_recipes.html');
		}
	}

	function _mergeAssoc($array1, $array2, $key) {
		$result = array();

		foreach ($array1 as $a1) {
			if (!$this->_hasKeyVal($array2, $key, $a1[$key])) {
				$result[] = $a1;
			}
		}

		return array_merge($result, $array2);
	}

	function _hasKeyVal($array, $key, $value) {
		foreach ($array as $a) {
			if ($a[$key] == $value) {
				return true;
			}
		}

		return false;
	}
}

?>