<?php

class UnitTest_Controller extends AbstractController
{
	private $_current_test;
	function index()
	{
		ini_set('display_errors', 'stdout');
		error_reporting(E_ALL);

		$this->load->model('Recipe_Model', 'rm');
		$this->load->model('Special_Model', 'sm');
		$this->load->model('Favorites_Model', 'fm');

		$this->_test('Recipe Model Available', ($this->rm != null), '');
		$this->_test('Special Model Available', ($this->sm != null), '');
		$this->_test('Favorites Model Available', ($this->fm != null), '');
		$this->_test('Database Object Available', ($this->rm->db != null), '');

		// run unit tests on our models.
		$this->test_unit_test();
		$this->test_view();
		$this->test_recipe_model();
		$this->test_special_model();
		$this->test_user_model();
		$this->test_favorites_model();

		$this->view->display('unittest/unittest.html');
	}

	function test_unit_test()
	{
		// check that we have the necessary requirements for unit testing.
	}

	function test_recipe_model()
	{
		$recipe_name = 'yogurt';
		$ingredient_name = 'salt';
		$recipe_id = 1;
		$special_id = 1;

		$this->_printr($this->rm->getRecipesByName($recipe_name), "Example recipe results [$recipe_name]");

		$this->_test(
			'test_recipe_model#getRecipesByName',
			$this->_notempty($this->rm->getRecipesByName($recipe_name)),
			"No recipes matching search term '$recipe_name'"
		);

		$this->_test(
			'test_recipe_model#getRecipesByIngredient',
			$this->_notempty($this->rm->getRecipesByIngredient($ingredient_name)),
			'No recipes matching ' . $ingredient_name
		);

		$this->_test(
			'test_recipe_model#getRecipeById',
			$this->rm->getRecipeById($recipe_id),
			'No result for recipeid ' . $recipe_id
		);

		$this->_test(
			'test_recipe_model#getRecipesForSpecial',
			$this->_notempty($this->rm->getRecipesForSpecial($special_id)),
			'No recipes for specialid ' . $special_id
		);
	}

	function test_special_model()
	{
		$this->_printr($this->sm->getSpecialsByName('Water'), 'Example special results.');

		$this->_test(
			'test_special_model#getSpecialsByName',
			$this->_notempty($this->sm->getSpecialsByName('Water')),
			'No specials matching Water.'
		);

		$this->_test(
			'test_special_model#getSpecialsByName with zipcode',
			$this->_notempty($this->sm->getSpecialsByName('Water', 98072)),
			'No specials matching Water with zipcode 98072.'
		);

		$this->_test(
			'test_special_model#getSpecialsByIngredient',
			$this->_notempty($this->sm->getSpecialsByIngredient('yogurt')),
			'No specials for ingredient yogurt'
		);

		$this->_test(
			'test_special_model#getSpecialsByArea',
			$this->_notempty($this->sm->getSpecialsByArea('98072')),
			'No specials for zipCode 98072'
		);

		$this->_test(
			'test_special_model#getSpecialById',
			$this->_notempty($this->sm->getSpecialById(1)),
			'No matching special for specialid 1'
		);

		$this->_test(
			'test_special_model#getSpecialsForRecipe',
			$this->_notempty($this->sm->getSpecialsForRecipe(1)),
			'No specials for recipeid 1'
		);
	}

	function test_user_model()
	{
		// todo: implement user model tests.
	}

	function test_favorites_model()
	{

		$this->_test(
			'test_favorites_model#getSavedFoods',
			$this->_notempty($this->fm->getSavedFoods(1)),
			'No saved foods for userid 1'
		);

		$this->_test(
			'test_favorites_model#getSavedRecipes',
			$this->_notempty($this->fm->getSavedRecipes(1, $this->rm)),
			'No saved recipes for userid 1'
		);

		$this->_test(
			'test_favorites_model#getSavedSpecials',
			$this->_notempty($this->fm->getSavedSpecials(1, $this->sm)),
			'No saved specials for userid 1'
		);

		$userid = 1;
		$food = 'My Food';
		$recipe = 2;
		$special = 5;
		
		$this->fm->addFood($userid, $food);
		$this->_test(
			'test_favorites_model#addFood',
			array_search('My Food', $this->fm->getSavedFoods($userid)),
			"No insert id for addFood [{$this->fm->db->lastInsertId()}]"
		);

		$this->fm->addRecipe($userid, $recipe);
		$this->_test(
			'test_favorites_model#addRecipe',
			array_search($recipe, $this->fm->getSavedRecipeIds($userid)),
			"No insert id for addRecipe [{$this->fm->db->lastInsertId()}]"
		);

		$this->fm->addSpecial($userid, $special);
		$this->_printr($this->fm->getSavedSpecialIds($userid), 'Saved Special Ids after add.');
		$this->_test(
			'test_favorites_model#addSpecial',
			array_search($special, $this->fm->getSavedSpecialIds($userid)),
			"No insert id for addSpecial [{$this->fm->db->lastInsertId()}]"
		);

		$this->fm->removeFood($userid, $food);
		$this->_test(
			'test_favorites_model#removeFood',
			!array_search($food, $this->fm->getSavedFoods($userid)),
			'Not removed'
		);

		$this->fm->removeRecipe($userid, $recipe);
		$this->_test(
			'test_favorites_model#removeRecipe',
			!array_search($recipe, $this->fm->getSavedRecipeIds($userid)),
			'Not removed'
		);

		$this->fm->removeSpecial($userid, $special);
		$this->_test(
			'test_favorites_model#removeSpecial',
			!array_search($special, $this->fm->getSavedSpecialIds($userid)),
			'Not removed'
		);
	}

	function test_view()
	{
		$this->view->assign('unittest', 'xyzxyz');
		$this->view->assign('ut_items', array('FRANKLIN', 'ROOSEVELT', 'PRESIDENT'));
		$output = $this->view->fetch('unittest/unittest_test_view.html');

		$this->_test(
			'test_view#blank output',
			trim($output) != '',
			'View output was blank.'
		);

		$this->_test(
			'test_view#assign',
			strstr($output, 'xyzxyz'),
			'Smarty Assign not working'
		);

		$this->_test(
			'test_view#foreach loop',
			strstr($output, 'FRANKLIN') && strstr($output, 'PRESIDENT'),
			'Smarty {foreach} not working'
		);
	}

	function _test($testName, $condition, $errorMessage)
	{
		if ($condition) {
			$this->_result($testName, $condition, '');
		} else {
			$this->_result($testName, $condition, $errorMessage);
			$this->_error($testName, $errorMessage);
		}
	}
	function _empty($array)
	{
		return count($array) == 0;
	}
	function _notempty($array)
	{
		return $array != null && is_array($array) && count($array) > 0;
	}
	// helper function used to add a test and its result to the final output.
	function _result($test, $passed, $message)
	{
		$this->view->append('tests', array(
			'name' => $test,
			'passed' => $passed,
			'message' => $message,
			));
	}

	// helper function to print the content of an object or array.
	function _printr($object, $message = '')
	{
		if (Url::getBool('verbose')) {
			$this->view->append('outputs',
				array(
					'object' => print_r($object, true),
					'message' => $message
				)
			);
		}
	}

	// helper function used to add an error to the final output.
	function _error($testName, $error)
	{
		$this->view->append('errors', "Error in [{$testName}] test. [{$error}]");
	}
}

?>