<?php

class Recipe_Model extends AbstractModel
{
	function getRecipesByName($query)
	{
		$query = $this->db->getLike($query);
		$recipes = $this->db->queryAll('select id, name, directions from recipe where name LIKE ?', array($query));

		$recipes = $this->_joinIngredients($recipes);

		return $recipes;
	}

	function getRecipesByIngredient($query)
	{
		$query = $this->db->getLike($query);

		// get recipes with ingredients matching query.
		$recipes = $this->db->queryAll('
			select distinct r.id, r.name, r.directions
			from recipe r, recipeingredient ri, ingredient i
			where r.id = ri.recipeId and ri.ingredientId = i.id
				and i.name like ?', array($query));

		$recipes = $this->_joinIngredients($recipes);

		return $recipes;
	}

	function getRecipeById($id)
	{
		$recipe = $this->db->queryRow('
			select id, name, directions
			from recipe
			where id = ?', array($id));
		$recipe['ingredients'] = $this->_getIngredients($id);

		return $recipe;
	}

	function getRecipesForSpecial($specialId)
	{
		// get recipes with ingredients mentioned in special
		$recipes = $this->db->queryAll('
			select distinct r.id, r.name, r.directions
			from recipe r, specialingredient si, recipeingredient ri
			where si.ingredientId = ri.ingredientId and ri.recipeId = r.id
				and si.specialId = ?', array($specialId));

		$recipes = $this->_joinIngredients($recipes);
		return $recipes;
	}

	function _joinIngredients($recipes) {
		$result = array();
		foreach ($recipes as $recipe) {
			$recipe['ingredients'] = $this->_getIngredients($recipe['id']);
			$result[] = $recipe;
		}

		return $result;
	}

	function _getIngredients($recipeId) {
		$ingredients = $this->db->queryAll('
			select ri.recipeId, ri.ingredientId, i.name, ri.quantity, ri.preparation
			from recipeingredient ri, ingredient i
			where ri.recipeId = ? and ri.ingredientId = i.id', array($recipeId));

		return $ingredients;
	}
}

?>